# This is the project for checking excels about cosmic related projects/folders

from typing import NamedTuple, Literal
class Version(NamedTuple):
    major: int
    minor: int
    micro: int
    releasetype: Literal['alpha', 'beta', 'stable']
    serial: int

version : Version = Version(major=0, minor=2, micro=2, releasetype='stable', serial=0)



# This file is used to find possible files by requirement

from cosmicexcelchecker._baseclass import UnionExcels

import glob

class FindExcels(UnionExcels):
    '''
    Concrete class for finding all possible Excels under certain paths
    '''

    @staticmethod
    def path_format(path: str) -> str:
        # improve format of path
        return path.replace('\\', '/').strip()

    @staticmethod
    def find_excels(path: str) -> list[str, None]:
        path = FindExcels.path_format(path=path)

        if not path.endswith('/**/*'):
            path += '/**/*'  # to iterate all files

        # using glob for recursive searching
        excels : list[str, None] = []
        for filename in glob.iglob(pathname=path, recursive=True):
            if filename.endswith(('.xlsx', '.xls', '.csv')):
                excels.append(filename.replace('\\', '/'))

        return excels


# Core COSMIC File

from cosmicexcelchecker._baseclass import PdExcel
from typing import Union, Dict, List
from cosmicexcelchecker.errors import CosmicExcelCheckerException ,IncorrectFileTypeException, RepeatedREQNumException, \
    SheetNotFoundException, UnknownREQNumException
from tabulate import tabulate
from cosmicexcelchecker.conf import CFP_SHEET_NAMES ,CFP_COLUMN_NAME, SUB_PROCESS_NAME, RS_SKIP_ROWS, RS_TOTAL_CFP_NAME, \
    RS_WORKLOAD_NAME, RS_REQ_NUM, RS_REQ_NAME, SR_COSMIC_REQ_NAME,  SR_NONCOSMIC_REQ_NAME, SR_SUBFOLDER_NAME, \
    SR_COSMIC_FILE_PREFIX, SR_NONCOSMIC_FILE_PREFIX, RS_QLF_COSMIC, COEFFICIENT_SHEET_NAME, \
    COEFFICIENT_SHEET_DATA_COL_NAME, NONCFP_SHEET_NAMES, SR_NONCOSMIC_PROJECT_NAME, SR_NONCOSMIC_REQ_NUM, \
    SR_AC_REPORT_NUM, SR_AC_FINAL_NUM, SR_FINAL_CONFIRMATION, SR_AC_REQ_NUM, SR_AC_REQ_NAME, SR_AC_FINAL_NUM_LIMIT

from .find import FindExcels

import pandas as pd
import numpy as np
import time
import re
import math
import xlrd
import openpyxl

class CosmicReqExcel(PdExcel):
    '''
    Implementation of abstract class PdExcel
    Read a single Excel/CSV about Cosmic Requirement
    '''

    def __init__(self, path: str):
        '''
        data_frames is the pd.DataFrame converted from Spreadsheet
        log holds temporary error/warning for later usage (e.g print to terminal)

        :param path: path to single requirement file
        '''
        self.path : str = path
        self.data_frames: Union[Dict[str, pd.DataFrame], None] = None
        self.log : Union[List[str], str, None] = None
        self.file_format : Union[str, None] = None

    def load_excel(self):
        file_ext = self.path[self.path.rindex('.'):]

        # it can be simplified to a dict[file_ext:engine] but with less readability
        if file_ext in ('.xlsx', '.xls'):
            self.data_frames = pd.read_excel(self.path, sheet_name=None)

            self.file_format = file_ext
        else:
            raise IncorrectFileTypeException(f"{self.path} is not a valid relative file path for an Excel file")

    def print_df(self):
        '''
        Try to print the converted pd.Dataframe to the terminal

        :return: None
        '''

        for df in self.data_frames.values() if isinstance(self.data_frames, dict) else self.data_frames:

            print(tabulate(df, headers='keys', tablefmt='psql'))

    def get_req_name(self) -> Union[str, None]:
        '''
        Get the requirement name of a single req file

        :return: name string or None if not no data found
        '''

        cfp_df : Union[pd.DataFrame, None] = None
        for sheet_name in CFP_SHEET_NAMES:  # iterate through
            cfp_df = self.data_frames.get(sheet_name, None) if isinstance(self.data_frames, dict) else None

            if cfp_df is not None:
                break

        if cfp_df is None:  # noqa
            return None

        # get column name by iterating through .columns
        for col_name in cfp_df.columns:
            if col_name.startswith(SR_COSMIC_REQ_NAME):
                return cfp_df.iloc[0, cfp_df.columns.get_loc(col_name)]

        return None


    def get_CFP_total(self) -> Union[float, None]:
        '''
        get total CFP pts under CFP column
        it will convert all possible numeric values to dtype float (or int) and leave others as NaN

        :return: total CFP pts
        '''
        data : Union[pd.DataFrame, None] = None
        for sheet_name in CFP_SHEET_NAMES:  # iterate through
            data = self.data_frames.get(sheet_name, None) if isinstance(self.data_frames, dict) else None

            if data is not None:
                break

        if data is None or CFP_COLUMN_NAME not in data.columns or SUB_PROCESS_NAME not in data.columns:
            return None

        cfp_s : pd.Series = pd.to_numeric(data.loc[:, CFP_COLUMN_NAME], errors='coerce')  # convert to numeric
        cfp_df : pd.DataFrame = data.loc[:, [SUB_PROCESS_NAME, CFP_COLUMN_NAME]]

        sub_process_null_count = 0  # only when subprocess is null, not both null
        for index, row in cfp_df.iterrows():
            if pd.isna(row[SUB_PROCESS_NAME]) and not pd.isna(row[CFP_COLUMN_NAME]):
                sub_process_null_count += 1

        return cfp_s.sum() - sub_process_null_count

    def check_CFP_column(self) -> dict:
        '''
        check cfp column with incorrect value
        Correct values: [0, 1]

        :return: a dict-format result
        '''

        for sheet_name in CFP_SHEET_NAMES:  # iterate through
            cfp_df : Union[pd.DataFrame, None] = self.data_frames.get(sheet_name, None) if isinstance(self.data_frames, dict) else None

            if cfp_df is not None:
                break

        if cfp_df is None or CFP_COLUMN_NAME not in cfp_df or SUB_PROCESS_NAME not in cfp_df:  # noqa
            return {'path': self.path, 'match': False, 'CFP': -1, 'note': 'No CFP related column'}

        cfp_df : pd.DataFrame = cfp_df.loc[:, [SUB_PROCESS_NAME, CFP_COLUMN_NAME]]  # noqa
        cfp_total = self.get_CFP_total()  # it will always return a float since CFP column exists

        # record if cfp or sub-process column miss anything
        note = f"Missing data in {'CFP Column' * int(cfp_df.loc[:, CFP_COLUMN_NAME].isna().sum() > 0)} " \
               f"{'Subprocess description' * int(cfp_df.loc[:, SUB_PROCESS_NAME].isna().sum() > 0)}".strip()
        # if cfp_df.loc[:, CFP_COLUMN_NAME].isna().sum() > 0:  # blank space or non-numeric char
        #     note = 'Missing data in CFP Column'
        #
        # if cfp_df.loc[:, SUB_PROCESS_NAME].isna().sum() > 0:
        #     note +=
        if note == 'Missing data in':  # meaning there's no missing
            note = ''

        return {
            "path": self.path,
            "match": True,
            "CFP": cfp_total,
            "note": note
        }

    def check_coefficient_sheet(self) -> Union[bool, None]:
        '''
        check CFP pts is the same with the B1 CFP pts in coefficient sheet if applicable

        :return: match/not match (bool) or None if there's no coefficient_sheet
        '''

        coefficient_sheet: pd.DataFrame = self.data_frames.get(COEFFICIENT_SHEET_NAME, None)

        if coefficient_sheet is None:
            return None

        std_cfp_pts = coefficient_sheet.iloc[1, coefficient_sheet.columns.get_loc(COEFFICIENT_SHEET_DATA_COL_NAME)]

        if std_cfp_pts is None or std_cfp_pts == '':
            return None

        try:
            std_cfp_pts = float(std_cfp_pts)

            return std_cfp_pts == self.get_CFP_total()

        except ValueError:
            return None

    def check_final_confirmation(self) -> dict:
        '''
        check final confirmation worksheet if applicable, by the contents comparing to itself and other sheets

        :return: a result dictionary shows the match info
        '''
        for sheet_name in SR_FINAL_CONFIRMATION:
            fc_sheet : Union[pd.DataFrame, None] = self.data_frames.get(sheet_name, None) if \
                isinstance(self.data_frames, dict) else None

            if fc_sheet is not None:
                break

        if fc_sheet is None:  # noqa
            return {
                "path": self.path,
                "match": False,
                "note": "No related final confirmation worksheet found"
            }

        # ignore first row, set index 1 as column and reset index.
        fc_sheet.columns = fc_sheet.iloc[0]  # noqa
        fc_sheet = fc_sheet.iloc[1:]
        fc_sheet = fc_sheet.reset_index(drop=True)

        # load other sheets for comparison
        coefficient_sheet : pd.DataFrame = self.data_frames.get(COEFFICIENT_SHEET_NAME, None) \
            if isinstance(self.data_frames, dict) else None

        if coefficient_sheet is None:
            return {
                "path": self.path,
                "match": False,
                "note": "No related coeffficient worksheet found"
            }

        for sheet_name in CFP_SHEET_NAMES:  # iterate through
            cfp_df : Union[pd.DataFrame, None] = self.data_frames.get(sheet_name, None) \
                if isinstance(self.data_frames, dict) else None

        if cfp_df is None:  # noqa
            return {
                "path": self.path,
                "match": False,
                "note": "No related Cosmic worksheet found"
            }

        try:
            note : str = ''

            # extract folder req by path
            extract_folder_req = rf"\/(?P<req>[0-9]{{1,4}})\/{SR_SUBFOLDER_NAME}\/{SR_COSMIC_FILE_PREFIX}[0-9A-Za-z,&@#$%.\[\]{{}};'\u4e00-\u9fff：。，（）()’￥……]+\.xls[x]{{0,1}}$"
            match = ""
            for m in re.finditer(pattern=extract_folder_req, string=self.path):
                match = m['req']  # type: str

            if match == '':
                note += 'Path Corrupted (no req num subfolder)\t'

            if not match.isnumeric():
                note += 'Missing Req num (cosmic, fc)'

            elif fc_sheet.iloc[0, fc_sheet.columns.get_loc(SR_AC_REQ_NUM)] != int(match):
                    note += 'Req Num not match (cosmic, fc)\t'

            # check req name
            if fc_sheet.iloc[0, fc_sheet.columns.get_loc(SR_AC_REQ_NAME)] != self.get_req_name():
                note += 'Req Name not match (cosmic, fc)\t'

            # **DO NOT DELETE**
            # # check report num of days
            # std_num_days = round(coefficient_sheet.iloc[-1, -1])
            # if abs(fc_sheet.iloc[0, fc_sheet.columns.get_loc(SR_AC_REPORT_NUM)] - std_num_days) >= 0.1:  # errors
            #     note += 'report num of days not match (coefficient, fc)\t'
            #
            # # check final report num of days
            # if std_num_days <= SR_AC_FINAL_NUM_LIMIT:
            #     final_num_days = std_num_days
            # else:
            #     final_num_days = SR_AC_FINAL_NUM_LIMIT
            #
            # fnd_inchart = fc_sheet.iloc[0, fc_sheet.columns.get_loc(SR_AC_FINAL_NUM)]
            # if fnd_inchart > SR_AC_FINAL_NUM_LIMIT or abs(fnd_inchart - final_num_days) >= 0.1:  # errors
            #     note += 'final num of days not match (fc)\t'
            #
            # note = note.strip('\t')

            return {
                "path": self.path,
                "match": note == '',
                "note": note
            }

        except KeyError:
            return {
                "path": self.path,
                "match": False,
                "note": "Key Error in worksheet. Make sure they are in standard format"
            }

    def check_highlight_cfp(self) -> list[int, None]:
        '''
        check the highlight on sub-process and its corresponding cfp in the same line
        No fill: 1 cfp; Yellow: 0 cfp; Red: 1/3 cfp

        :return:
        '''

        # load the Excel again using openpyxl/xlrd
        if self.file_format == '.csv':
            return list()
        elif self.file_format == '.xlsx':
            excel = openpyxl.load_workbook(self.path)

            # open cfp sheet
            sheet = None
            cfp_df : Union[pd.DataFrame, None] = None
            for sheet_name in CFP_SHEET_NAMES:
                if sheet_name in excel:
                    sheet = excel[sheet_name]
                    cfp_df = self.data_frames.get(sheet_name, None) if \
                        isinstance(self.data_frames, dict) else None
                    break

            if sheet is None or cfp_df is None:
                raise SheetNotFoundException(f"Standard CFP Sheet not found")

            # get subprocess index and cfp
            sp_idx = cfp_df.columns.get_loc(SUB_PROCESS_NAME)  # subprocess col
            cfp_idx = cfp_df.columns.get_loc(CFP_COLUMN_NAME)

            err_list : list = []
            row_num = 1  # 1-based, =1 since min_row = 2
            # extract cell value and compare
            for row in sheet.iter_rows(min_row=2, max_row=len(cfp_df.index) + 1, min_col=sp_idx + 1, max_col=sp_idx + 1):  # 1-based idx for min_row, each row is a tuple
                row_num += 1
                sp_color_hex : str = ""
                cfp_cell : str = ""
                # for i in range(len(row)):
                #     if i == sp_idx and row[i].value != SUB_PROCESS_NAME:  # avoid first row
                #         sp_color_hex : str = row[i].fill.start_color.index  # hex value for the cell fill color
                #
                #     if i == cfp_idx and row[i].value != CFP_COLUMN_NAME:
                #         cfp_cell = str(row[i].value)
                sp_color_hex : str = row[0].fill.start_color.index
                cfp_cell = str(cfp_df.iloc[row_num - 2, cfp_idx])  # avoid str cell value

                if sp_color_hex == "":  # not counted since subprocess is empty
                    continue

                if cfp_cell == "":  # only count valid subprocess row
                    err_list.append(f'{row_num} Missing Data')
                    continue
                try:
                    cfp_cell : float = float(cfp_cell)
                except ValueError:
                    err_list.append(f'{row_num} CFP not a number')
                    continue

                if sp_color_hex == 'FFFFFF00' and cfp_cell != 0:  # YELLOW
                    err_list.append(f'{row_num} Yellow != 0')
                elif sp_color_hex == 'FFFF0000' and abs(cfp_cell - 1/3) >= 0.01:  # RED
                    err_list.append(f'{row_num} Red != 1/3 or 0.333')
                elif (sp_color_hex == '00000000' or (type(sp_color_hex) is int and sp_color_hex == 9)) and cfp_cell != 1:  # No fill
                    err_list.append(f'{row_num} No fill (White) != 1')

        elif self.file_format == '.xls':
            excel = xlrd.open_workbook(self.path, formatting_info=True)

            # open cfp sheet
            sheet = None
            cfp_df: Union[pd.DataFrame, None] = None
            for sheet_name in CFP_SHEET_NAMES:
                if sheet_name in excel.sheet_names():
                    sheet = excel.sheet_by_name(sheet_name)
                    cfp_df = self.data_frames.get(sheet_name, None) if \
                        isinstance(self.data_frames, dict) else None
                    break

            if sheet is None or cfp_df is None:
                raise SheetNotFoundException(f"Standard CFP Sheet not found")

            # get idx of subprocess and cfp col
            sp_idx = cfp_df.columns.get_loc(SUB_PROCESS_NAME)  # subprocess col
            cfp_idx = cfp_df.columns.get_loc(CFP_COLUMN_NAME)

            err_list = []
            rows = min(sheet.nrows, len(cfp_idx.index) + 1)

            for i in range(1, rows):
                sp_cell : xlrd.sheet.Cell = sheet.cell(rowx=i, colx=sp_idx)
                # cfp_cell : xlrd.sheet.Cell = sheet.cell(rowx=i, colx=cfp_idx)

                xfx = sheet.cell_xf_index(rowx=i, colx=sp_idx)  # xf index
                xf = excel.xf_list[xfx]  # used xfx as index
                bgx = xf.background.pattern_colour_index

                cfp : str = str(cfp_df.iloc[i - 1, cfp_idx])
                if str(sp_cell.value) == "":  # not count as valid if subprocess is empty
                    continue

                if cfp == "":  # only count valid subprocess row
                    err_list.append(f'{i + 1} Missing data')
                    continue

                try:
                    cfp : float = float(cfp)
                except ValueError:
                    err_list.append(f'{i + 1} CFP not a number')
                    continue

                if bgx == 13 and float(cfp) != 0:  # YELLOW
                    err_list.append(f'{i + 1} Yellow != 0')
                elif bgx == 10 and abs(float(cfp) - 1/3) >= 0.01:  # RED
                    err_list.append(f'{i + 1} Red != 1/3 or 0.333')
                elif bgx == 64 and float(cfp) != 1:  # no fill
                    err_list.append(f'{i + 1} No fill (White) != 1')

        else:
            raise IncorrectFileTypeException(f"Incorrect file type {self.file_format}. It has to be .xlsx or .xls file (.csv deprecated)")

        return err_list

class NonCosmicReqExcel(PdExcel):
    '''
    Implementation of Abstract class PdExcel
    Mainly used for representing non-cosmic Excel file (single requirement)
    '''

    def __init__(self, path: str):
        '''
                data_frames is the pd.DataFrame converted from Spreadsheet
                log holds temporary error/warning for later usage (e.g print to terminal)

                :param path: path to single requirement file
                '''
        self.path: str = path
        self.data_frames: Union[Dict[str, pd.DataFrame], None] = None
        self.log: Union[List[str], str, None] = None

    def load_excel(self):
        file_ext = self.path[self.path.rindex('.'):]

        # it can be simplified to a dict[file_ext:engine] but with less readability
        if file_ext in ('.xlsx', '.xls'):
            self.data_frames = pd.read_excel(self.path, sheet_name=None)
        else:
            raise IncorrectFileTypeException(f"{self.path} is not a valid relative file path for an Excel file")

    def print_df(self):
        '''
        Try to print the converted pd.Dataframe to the terminal

        :return: None
        '''

        for df in self.data_frames.values() if isinstance(self.data_frames, dict) else self.data_frames:
            print(tabulate(df, headers='keys', tablefmt='psql'))

    def get_req_name(self) -> Union[str, None]:
        '''
        Get requirement name for the Excel

        :return: requirement as a string or None if no related column found
        '''

        workload_df = self.data_frames.get(NONCFP_SHEET_NAMES, None)

        if workload_df is None or SR_NONCOSMIC_REQ_NAME not in workload_df.columns:
            return None

        try:
            return workload_df.iloc[0, workload_df.columns.get_loc(SR_NONCOSMIC_REQ_NAME)]
        except IndexError:  # if not exists
            return None

    def get_project_name(self) -> Union[str, None]:
        '''
        Get project name for the Excel

        :return: requirement as a string or None if no related column found
        '''

        workload_df = self.data_frames.get(NONCFP_SHEET_NAMES, None)

        if workload_df is None or SR_NONCOSMIC_PROJECT_NAME not in workload_df.columns:
            return None

        try:
            return workload_df.iloc[0, workload_df.columns.get_loc(SR_NONCOSMIC_PROJECT_NAME)]
        except IndexError:  # if not exists
            return None

class ResultSummary(PdExcel):
    '''
    Another implementation of Abstract class PdExcel
    Demonstrated for loading and processing data in Result Summary related excels
    '''

    def __init__(self, path: str, folders_path:str, sheet_name: str):
        '''
        data_frames is the pd.DataFrame converted from Spreadsheet
        log holds temporary error/warning for later usage (e.g print to terminal)

        :param path: path of the result summary file
        :param sheet_name: specific worksheet name in result summary to be loaded
        '''
        self.path : str = FindExcels.path_format(path=path)
        self.folders_path : str = FindExcels.path_format(path=folders_path)
        self.data_frames: Union[Dict[str, pd.DataFrame], None] = None
        self.data_frame_specific : Union[pd.DataFrame, None] = None
        self.log : Union[list[str], str, None] = None
        self.sheet_name : str = sheet_name
        self.file_paths : Union[list[str, None], None] = FindExcels.find_excels(path=self.folders_path)

    def load_excel(self):
        file_ext = self.path[self.path.rindex('.'):]

        # it can be simplified to a dict[file_ext:engine] but with less readability
        if file_ext in ('.xlsx', '.xls'):
            self.data_frames = pd.read_excel(self.path, sheet_name=None, skiprows=range(RS_SKIP_ROWS))

            df_specific = self.data_frames.get(self.sheet_name, None)

            if df_specific is None:
                raise SheetNotFoundException(f"Sheet with name {self.sheet_name} is not found inside the given file")

            self.data_frame_specific = df_specific
        else:
            raise IncorrectFileTypeException(f"{self.path} is not a valid relative file path for an Excel file")

    def set_sheet_name(self, sheet_name: str):
        '''
        Setting worksheet name for reading dataframe later

        :param sheet_name: str, name of the worksheet
        :return: None
        '''

        self.sheet_name = sheet_name

        df_specific = self.data_frames.get(self.sheet_name, None)

        if df_specific is None:
            raise SheetNotFoundException(f"Sheet with name {sheet_name} is not found inside the given file")

        self.data_frame_specific = df_specific


    def print_df(self):
        '''
        Try to print the converted pd.Dataframe to the terminal

        :return: None
        '''

        for df in self.data_frames.values() if isinstance(self.data_frames, dict) else self.data_frames:

            print(tabulate(df, headers='keys', tablefmt='psql'))


    def print_df_specific(self):
        '''
        Try to print the specific sheet set by function set_sheet_name
        :return: None
        '''

        if isinstance(self.data_frame_specific, pd.DataFrame):
            print(tabulate(self.data_frame_specific, headers='keys', tablefmt='psql'))

        else:
            raise CosmicExcelCheckerException("Specific worksheet is not loaded. Use `set_sheet_name` to load it")

    def check_ratio(self) -> list[str, None]:
        '''
        check all columns workload and cfp ratio, default to 0.79

        :return: a list contains all non-qualified requirements
        '''

        if self.data_frame_specific is None or RS_WORKLOAD_NAME not in self.data_frame_specific.columns or RS_TOTAL_CFP_NAME not in self.data_frame_specific.columns:
            return list()

        result_summary = self.data_frame_specific.loc[:, [RS_REQ_NUM, RS_REQ_NAME, RS_WORKLOAD_NAME,RS_TOTAL_CFP_NAME]]

        bad_ratio : list[str, None] = []  # record
        for index, row in result_summary.iterrows():
            if math.ceil(int(row[RS_WORKLOAD_NAME]) / 0.79) < int(row[RS_TOTAL_CFP_NAME]):
                bad_ratio.append(
                    f'{RS_REQ_NUM}{row[RS_REQ_NUM]}, {RS_REQ_NAME}: {row[RS_REQ_NAME]}'
                )

        return bad_ratio

    def check_file(self, req_num: int, check_final_confirmation: bool = True,
                   check_highlight_cfp: bool = True) -> dict:
        '''
        check a single file data, comparing to the result summary xlsx
        check req number, req name, CFP total, CFP Total comparison

        :param: req_num is the requirement number 需求序号
        :return: a dict-format result
        '''

        try:
            if self.data_frame_specific is None:
                raise CosmicExcelCheckerException()
        except CosmicExcelCheckerException:
            print("Specific worksheet is not loaded. Use `set_sheet_name` to load it")
            return dict()

        # get row indices qualified for req_num
        r_indices : list = self.data_frame_specific.index[self.data_frame_specific[RS_REQ_NUM] == req_num].tolist()

        try:
            if len(r_indices) < 0:
                raise UnknownREQNumException()

            elif len(r_indices) > 1:
                raise RepeatedREQNumException()

        except UnknownREQNumException:
            print(f"Sheet does not have a requirement number called {req_num}")
            return dict()

        except RepeatedREQNumException:
            print(f"Sheet has repeated rows for requirement number {req_num}")
            return dict()

        row_index = r_indices[0]

        req_num = self.data_frame_specific.iloc[row_index, self.data_frame_specific.columns.get_loc(RS_REQ_NUM)]

        # requirement folder path based on req_num
        req_folder_path_pattern = rf'{self.folders_path}\/(?:.*\/|){str(req_num)}\/'

        # qualified paths inside self.file_paths
        qualified_paths : list = [path for path in self.file_paths if re.match(pattern=req_folder_path_pattern, string=path)]

        if len(qualified_paths) == 0:  # no subfolder found
            return {"REQ Num": req_num, "path": "Not exist", "match": False, "note": "REQ folder does not exist"}

        qualified_cosmic = self.data_frame_specific.iloc[row_index, self.data_frame_specific.columns.get_loc(RS_QLF_COSMIC)]

        # check sr cosmic
        def check_cosmic(path: str):
            cosmic_excel = CosmicReqExcel(path=path)

            # load excel to class df
            cosmic_excel.load_excel()

            note = ''
            # check req name
            if self.data_frame_specific.iloc[
                row_index, self.data_frame_specific.columns.get_loc(RS_REQ_NAME)] != cosmic_excel.get_req_name():
                note += 'REQ name does not match (cosmic); '

            # check total CFP name
            total_cfp: str = str(
                self.data_frame_specific.iloc[row_index, self.data_frame_specific.columns.get_loc(RS_TOTAL_CFP_NAME)])
            if total_cfp.isnumeric():
                if float(total_cfp) != cosmic_excel.get_CFP_total():
                    note += 'Total CFP points do not match (cosmic); '
            else:
                note += 'CFP points in Result Summary is not valid (cosmic); '

            # Check coefficient sheet
            coefficient_sheet_match = cosmic_excel.check_coefficient_sheet()

            if coefficient_sheet_match is None:
                note += 'No Coefficient Sheet in Excel (cosmic); '
            elif coefficient_sheet_match is False:
                note += 'Coefficient Sheet B1 data does not match total standard CFP pts; '

            # check final confirmation worksheet
            if check_final_confirmation:
                fc_result = cosmic_excel.check_final_confirmation()
                if fc_result['note'] != "":
                    note += f"{fc_result['note']}; "

            # check highlight
            if check_highlight_cfp:
                hl_result = cosmic_excel.check_highlight_cfp()
                if hl_result != list():
                    note += f"highlight err: {str(hl_result)}; "

            note = note.rstrip('; ')

            return {"REQ Num": req_num, "path": path, "match": note == "", "note": note}

        # check sr noncosmic
        def check_noncosmic(path: str):
            noncosmic_excel = NonCosmicReqExcel(path=path)

            # load excel to class df
            noncosmic_excel.load_excel()

            note = ''
            # check req name
            if self.data_frame_specific.iloc[
                row_index, self.data_frame_specific.columns.get_loc(RS_REQ_NAME)] != noncosmic_excel.get_req_name():
                note += 'REQ name does not match (noncosmic); '

            # make sure cfp total is 0 for non-cosmic file
            # total_cfp: str = str(
            #     self.data_frame_specific.iloc[row_index, self.data_frame_specific.columns.get_loc(RS_TOTAL_CFP_NAME)])
            # if float(total_cfp) != cosmic_cfp:  # cosmic_cfp is total cfp checked by cosmic file
            #     note += 'Total CFP points is not 0 for non-cosmic requirement; '

            note = note.rstrip('; ')

            return {"REQ Num": req_num, "path": path, "match": note == "", "note": note}

        # qualified paths parent folder
        qualified_paths_docs = [path[path.rindex('/') + 1:] for path in qualified_paths]

        if qualified_cosmic == '是':
            if len(qualified_paths) == 1:

                if not qualified_paths_docs[0].startswith(SR_COSMIC_FILE_PREFIX):
                    return {"REQ Num": req_num, "path": qualified_paths[0], "match": False, "note": "Incorrect type of cosmic excel based on requirement"}

                # file matched
                try:
                    return check_cosmic(path=qualified_paths[0])
                except KeyError:
                    return {"REQ Num": req_num, "path": qualified_paths[0], "match": False, "note": "KeyError, Check column name"}

            else:
                return {"REQ Num": req_num, "path": qualified_paths[0], "match": False, "note": "Incorrect number of cosmic excel(s) based on requirement"}

        elif qualified_cosmic == '否':
            if len(qualified_paths) == 1:

                if not qualified_paths_docs[0].startswith(SR_NONCOSMIC_FILE_PREFIX):
                    return {"REQ Num": req_num, "path": qualified_paths[0], "match": False,
                            "note": "Incorrect type of cosmic excel based on requirement"}

                # file matched
                try:
                    return check_noncosmic(path=qualified_paths[0])
                except KeyError:
                    return {"REQ Num": req_num, "path": qualified_paths[0], "match": False,
                            "note": "KeyError, Check column name"}

            else:
                return {"REQ Num": req_num, "path": qualified_paths[0], "match": False,
                        "note": "Incorrect number of non-cosmic excel(s) based on requirement"}

        elif qualified_cosmic == '混合型':
            if len(qualified_paths) == 2:
                c_prefix_pattern = rf"{req_folder_path_pattern}(?:.*\/|){SR_SUBFOLDER_NAME}\/{SR_COSMIC_FILE_PREFIX}"
                nc_prefix_pattern = rf"{req_folder_path_pattern}(?:.*\/|){SR_SUBFOLDER_NAME}\/{SR_NONCOSMIC_FILE_PREFIX}"

                try:
                    if bool(re.match(c_prefix_pattern, qualified_paths[1])) and \
                            bool(re.match(nc_prefix_pattern, qualified_paths[0])):
                        c_result: dict = check_cosmic(path=qualified_paths[1])
                        nc_result : dict = check_noncosmic(path=qualified_paths[0])

                    elif bool(re.match(c_prefix_pattern, qualified_paths[0])) and \
                            bool(re.match(nc_prefix_pattern, qualified_paths[1])):
                        c_result : dict = check_cosmic(path=qualified_paths[0])
                        nc_result : dict = check_noncosmic(path=qualified_paths[1])
                    else:
                        return {"REQ Num": req_num, "path": qualified_paths[0], "match": False,
                                "note": "Incorrect type of cosmic/non-cosmic excel based on requirement"}

                    return {"REQ Num": req_num, "path": qualified_paths[0], "match": c_result['match'] & nc_result['match'],
                        "note": (c_result['note'] + '; ' + nc_result['note']).rstrip('; ')}

                except KeyError:
                    return {"REQ Num": req_num, "path": qualified_paths[0], "match": False, "note": "KeyError, Check column name"}

            else:
                return {"REQ Num": req_num, "path": qualified_paths[0], "match": False,
                        "note": "Incorrect number of cosmic/non-cosmic excel(s) based on requirement"}

        else:
            return {"REQ Num": req_num, "path": qualified_paths[0], "match": False,
                    "note": f"The parameter {qualified_cosmic} is not accepted"}

    def check_all_files(self, check_final_confirmation: bool = True,
                        check_highlight_cfp: bool = True) -> dict[str, list[dict, None]]:
        '''
        Check all related files listed in the result summary.
        Call `check_file` function for each single check.
        The time complexity is omega(n^2) for calling n items in the Excel of result summary.

        :param check_final_confirmation: bool for whether checking final confirmation, default to True
        :param check_highlight_cfp: bool for whether checking highlighting and corresponding cfp, default to True
        :return: A list of results in dict-format. Could be empty list if nothing found.
        '''

        if self.data_frame_specific is None:
            raise CosmicExcelCheckerException("Specific worksheet is not loaded. Use `set_sheet_name` to load it")

        start_time = time.time()
        # use iterrows() to iterate each row in DataFrame
        list_results : list[dict, None] = []
        for index, row in self.data_frame_specific.iterrows():
            list_results.append(self.check_file(
                req_num=row[RS_REQ_NUM],
                check_final_confirmation=check_final_confirmation,
                check_highlight_cfp=check_highlight_cfp
            ))

        cf_results = {
            "results": list_results,
            "time": round(time.time() - start_time, 5)
        }

        return cf_results




# CONFIG FILE TO CUSTOMIZE THE MODULE

# In COSMIC Requirement Spreadsheet, sheet name for demonstrating all CFP points
# default to ['功能点拆分表', 'COSMIC软件评估标准模板']
CFP_SHEET_NAMES = ['功能点拆分表', 'COSMIC软件评估标准模板']

# In NONCOSMIC Requirement Spreadsheet, Sheet name for illustration of workload
NONCFP_SHEET_NAMES = '非COSMIC评估工作量填写说明'

# In COSMIC Requirement Spreadsheet, Column Name for the CFP point, default to 'CFP'
CFP_COLUMN_NAME = 'CFP'

# Subprocess Name, default to '子过程描述'
SUB_PROCESS_NAME = '子过程描述'

# Result Summary Skiprows (结果反馈excel里跳过前..行), default to 9 (due to fixed format)
RS_SKIP_ROWS = 9

# Workload and CFP Ratio, default to 0.79
Workload_CFP_Ratio = 0.79

# Result Summary cosmic workload column name, default to 'cosmic送审工作量'
RS_WORKLOAD_NAME = 'cosmic送审工作量'

# Result Summary cosmic total cfp column name, default to 'cosmic送审功能点'
RS_TOTAL_CFP_NAME = 'cosmic送审功能点'

# Result Summary requirement number column name, default to '需求序号'
RS_REQ_NUM = '需求序号'

# Result Summary requirement name column name, default to '实施需求名称'
RS_REQ_NAME = '实施需求名称'

# Result Summary qualified cosmic column name, default to '是否适用cosmic'
RS_QLF_COSMIC = '是否适用cosmic'

# Single requirement (cosmic) name column name, default to 'OPEX-需求名称'
# This is NOT an EXACT string. This will be checked by startswith() for compatibility
SR_COSMIC_REQ_NAME = 'OPEX-需求名称'

# Single requirement (noncosmic) name column name, default to '需求名称'
SR_NONCOSMIC_REQ_NAME = '需求名称'

# Coefficient sheet name, default to '系数表'
COEFFICIENT_SHEET_NAME = '系数表'

# Coefficient sheet name data column, default to '数值'
COEFFICIENT_SHEET_DATA_COL_NAME = '数值'

# Requirement folder subfolder name, default to 'COSMIC评估发起'
SR_SUBFOLDER_NAME = 'COSMIC评估发起'

# Single Requirement Excel (cosmic) filename (prefix)
SR_COSMIC_FILE_PREFIX = '附件5'

# Single Requirement Excel (non-cosmic) filename (prefix)
SR_NONCOSMIC_FILE_PREFIX = '附件4'

# Single Requirement Excel (non-cosmic) req_num col name, default to '需求序号'
SR_NONCOSMIC_REQ_NUM = '需求序号'

# Single Requirement Excel (non-cosmic) project col name, default to '项目名称'
SR_NONCOSMIC_PROJECT_NAME = '项目名称'

# Single Requirement Excel (cosmic) analytics confirmation, default to
SR_FINAL_CONFIRMATION = ['结算评估确认表', '结论评估确认表', '结论认同表']

# Analytics confirmation (cosmic) req_num, default to '需求工单号'
SR_AC_REQ_NUM = '需求工单号'

# Analytics confirmation (cosmic) req_name, default to '需求名称'
SR_AC_REQ_NAME = '需求名称'

# Analytics confirmation (cosmic) report number (days), default to '上报工作量（人天）'
SR_AC_REPORT_NUM = '上报工作量\n（人天）'

# Analytics confirmation (cosmic) final result (days), default to '最终结果（人天）'
SR_AC_FINAL_NUM = '最终结果\n（人天）'

# Analytics confirmation (cosmic) final result (days)'s limitation, default to 3.4
SR_AC_FINAL_NUM_LIMIT = 3.4


# allow manually set config
def set_config(config: dict) -> None:
    '''
    Pass in a config dictionaries with {"conf name": value} to change the default value of the config

    :param config: a dict contains configuration
    :return: None
    '''

    if 'CFP_SHEET_NAMES' in config and isinstance(config['CFP_SHEET_NAMES'], (list, tuple)):
        global CFP_SHEET_NAMES
        CFP_SHEET_NAMES = config['CFP_SHEET_NAMES']

    if 'NONCFP_SHEET_NAMES' in config:
        global NONCFP_SHEET_NAMES
        NONCFP_SHEET_NAMES = config['NONCFP_SHEET_NAMES']

    if 'CFP_COLUMN_NAME' in config:
        global CFP_COLUMN_NAME
        CFP_COLUMN_NAME = config['CFP_COLUMN_NAME']

    if 'SUB_PROCESS_NAME' in config:
        global SUB_PROCESS_NAME
        SUB_PROCESS_NAME = config['SUB_PROCESS_NAME']

    if 'RS_SKIP_ROWS' in config:
        global RS_SKIP_ROWS
        RS_SKIP_ROWS = config['RS_SKIP_ROWS']

    if 'Workload_CFP_Ratio' in config:
        global Workload_CFP_Ratio
        Workload_CFP_Ratio = config['Workload_CFP_Ratio']

    if 'RS_WORKLOAD_NAME' in config:
        global RS_WORKLOAD_NAME
        RS_WORKLOAD_NAME = config['RS_WORKLOAD_NAME']

    if 'RS_TOTAL_CFP_NAME' in config:
        global RS_TOTAL_CFP_NAME
        RS_TOTAL_CFP_NAME = config['RS_TOTAL_CFP_NAME']

    if 'RS_REQ_NUM' in config:
        global RS_REQ_NUM
        RS_REQ_NUM = config['RS_REQ_NUM']

    if 'RS_REQ_NAME' in config:
        global RS_REQ_NAME
        RS_REQ_NAME = config['RS_REQ_NAME']

    if 'RS_QLF_COSMIC' in config:
        global RS_QLF_COSMIC
        RS_QLF_COSMIC = config['RS_QLF_COSMIC']

    if 'SR_COSMIC_REQ_NAME' in config:
        global SR_COSMIC_REQ_NAME
        SR_COSMIC_REQ_NAME = config['SR_COSMIC_REQ_NAME']

    if 'SR_NONCOSMIC_REQ_NAME' in config:
        global SR_NONCOSMIC_REQ_NAME
        SR_NONCOSMIC_REQ_NAME = config['SR_NONCOSMIC_REQ_NAME']

    if 'COEFFICIENT_SHEET_NAME' in config:
        global COEFFICIENT_SHEET_NAME
        COEFFICIENT_SHEET_NAME = config['COEFFICIENT_SHEET_NAME']

    if 'COEFFICIENT_SHEET_DATA_COL_NAME' in config:
        global COEFFICIENT_SHEET_DATA_COL_NAME
        COEFFICIENT_SHEET_DATA_COL_NAME = config['COEFFICIENT_SHEET_DATA_COL_NAME']

    if 'SR_SUBFOLDER_NAME' in config:
        global SR_SUBFOLDER_NAME
        SR_SUBFOLDER_NAME = config['SR_SUBFOLDER_NAME']

    if 'SR_COSMIC_FILE_PREFIX' in config:
        global SR_COSMIC_FILE_PREFIX
        SR_COSMIC_FILE_PREFIX = config['SR_COSMIC_FILE_PREFIX']

    if 'SR_NONCOSMIC_FILE_PREFIX' in config:
        global SR_NONCOSMIC_FILE_PREFIX
        SR_NONCOSMIC_FILE_PREFIX = config['SR_NONCOSMIC_FILE_PREFIX']

    if 'SR_NONCOSMIC_REQ_NUM' in config:
        global SR_NONCOSMIC_REQ_NUM
        SR_NONCOSMIC_REQ_NUM = config['SR_NONCOSMIC_REQ_NUM']

    if 'SR_NONCOSMIC_PROJECT_NAME' in config:
        global SR_NONCOSMIC_PROJECT_NAME
        SR_NONCOSMIC_PROJECT_NAME = config['SR_NONCOSMIC_PROJECT_NAME']

    if 'SR_FINAL_CONFIRMATION' in config and isinstance(config['SR_FINAL_CONFIRMATION'], (list, tuple)):
        global SR_FINAL_CONFIRMATION
        SR_FINAL_CONFIRMATION = config['SR_FINAL_CONFIRMATION']

    if 'SR_AC_REQ_NUM' in config:
        global SR_AC_REQ_NUM
        SR_AC_REQ_NUM = config['SR_AC_REQ_NUM ']

    if 'SR_AC_REQ_NAME' in config:
        global SR_AC_REQ_NAME
        SR_AC_REQ_NAME = config['SR_AC_REQ_NAME']

    if 'SR_AC_REPORT_NUM' in config:
        global SR_AC_REPORT_NUM
        SR_AC_REPORT_NUM = config['SR_AC_REPORT_NUM ']

    if 'SR_AC_FINAL_NUM' in config:
        global SR_AC_FINAL_NUM
        SR_AC_FINAL_NUM = config['SR_AC_FINAL_NUM']

    if 'SR_AC_FINAL_NUM_LIMIT' in config:
        global SR_AC_FINAL_NUM_LIMIT
        SR_AC_FINAL_NUM_LIMIT = config['SR_AC_FINAL_NUM_LIMIT']


import aiohttp
import asyncio

async def main(url):

    headers = {
        'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7',
        'Accept-Encoding': 'gzip, deflate, br',
        'Accept-Language': 'zh-CN,zh;q=0.9',
        'Referer': 'https://github.com'
    }

    async with aiohttp.ClientSession(trust_env=True) as session:
        async with session.get(url, headers=headers) as resp:
            if resp.status != 200:
                return []

            l = resp.json()
            return l

# l = asyncio.run(main(url='https://raw.githubusercontent.com/Discord-AntiScam/scam-links/main/urls.json'))
# print(l)
# print(len(l))
# print(type(l))

import requests

headers = {
        'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7',
        'Accept-Encoding': 'gzip, deflate, br',
        'Accept-Language': 'zh-CN,zh;q=0.9',
        'Referer': 'https://github.com'
    }

resp = requests.get(url='https://raw.githubusercontent.com/Discord-AntiScam/scam-links/main/urls.json', headers=headers)
if resp.status_code == 200:
    lj = resp.json()
    print(len(lj), type(lj))
    print(lj)


# script related to obfuscation (fuzzy word)

from cosmicexcelchecker._baseclass import AbstractObf

class CheckObf(AbstractObf):
    '''
    Concrete implementation of the AbstractObf class
    Provide static method to compare two strings
    '''

    @staticmethod
    def compare(string1: str, string2: str) -> int:
        '''
        Compare the edit distance of two given strings.
        Use the Levenshtein distance algorithem, which allows edit, add, delete
        Calculate the step numbers

        :param string1: a string input
        :param string2: another string input
        :return: step numbers as int
        '''

        # use var to get string1 + 1and string2 length + 1 since a lot of usage later
        len_str1_add1 = len(string1) + 1
        len_str2_add1 = len(string2) + 1

        # create mem space for dp, 1D array for performance but think as 2d array
        len_dp_mem = len_str1_add1 * len_str2_add1
        dp_mem = [0 for _ in range(len_dp_mem)]

        # generate for (a_i, 0) and (0, b_j) cells
        for i in range(len_str1_add1):
            dp_mem[i] = i

        for j in range(0, len_dp_mem, len_str1_add1):
            dp_mem[j] = j // len_str1_add1

        # iterate through each cell, check the min based on 3 ways (see levenshtein distance formula)
        for i in range(1, len_dp_mem):  # skip first zero to improve perf
            if dp_mem[i] != 0:  # first line or first col in 2D
                continue

            dp_mem[i] = min(dp_mem[i - 1] + 1, dp_mem[i - len_str1_add1] + 1, dp_mem[i - len_str1_add1 - 1] + \
                int(string1[i % len_str1_add1 - 1] != string2[i // len_str1_add1 - 1]))

        return dp_mem[-1]  # return the bottom right if in 2D

    @staticmethod
    def similarity(string1: str, string2: str, ratio: float) -> bool:
        '''
        use compare() to generate distance, "(longer - compare()) / longer" to compare ratio
        If greater than ratio, means similar. Otherwise, it's not
        Use 2-digit rounding, meaning num >= 0.785 -> 0.79

        :param string1: First string input
        :param string2: Second string input
        :param ratio: Given ratio to compare, default to 0.79
        :return: True if similar, False if not similar by comparing the ratio
        '''

        ed : int = CheckObf.compare(string1=string1, string2=string2)

        return round((max(len(string1), len(string2)) - ed) / max(len(string1), len(string2)), 2) >= ratio






from setuptools import setup, find_packages

with open('./README.md', 'r', encoding='utf-8') as f:
    long_des = f.read()

setup(
    name="cosmicexcelchecker",
    version="0.2.2",
    license="GPL-3.0-only",
    author="TimG233",
    author_email="gaosh0830@gmail.com",
    maintainer="TimG233",
    description="A high flexibility package for checking cosmic-related excels under CMDI cosmic standards",
    long_description=long_des,
    long_description_content_type="text/markdown",
    packages=[
        "cosmicexcelchecker",
    ],
    keywords=["cosmic", "excel", "CMDI", "checker", "data"],
    classifiers=[
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
    install_requires=[
        "numpy~=1.25.0",
        "openpyxl~=3.1.2",
        "pandas~=2.0.0",
        "tabulate~=0.9.0",
        "xlrd~=2.0.0",
    ],
    platforms="any",
    python_requires=">=3.9"
)




# _baseclass to include all abstract class for the module

from abc import ABC, abstractmethod

class PdExcel(ABC):
    '''
    An Abstract class demonstrated the base class related to the excel_files loaded
    All related file with concrete methods will implement this base class
    '''

    @abstractmethod
    def __init__(self, path: str):
        '''
        Instantiation of the class, which is an abstract method in this case
        :param path: RELATIVE path to the Excel file, file extension xlsx is needed
        '''

        pass

    @abstractmethod
    def load_excel(self):
        '''
        Load excel all possible spreedsheets from Excel file
        :return: None
        '''

        pass

class UnionExcels(ABC):
    '''
    An abstract class designed for finding all Excel files under certain directory
    This will be implemented by classes with concrete methods
    '''

    @staticmethod
    @abstractmethod
    def path_format(path: str) -> str:
        '''
        format path by replacing '\' or '\\' to '/'
        :param path:
        :return: formatted path
        '''

        pass

    @staticmethod
    @abstractmethod
    def find_excels(path: str) -> list[str, None]:
        '''
        find all possible Excel files under path (in constructor) and return name of them as a list
        :path: relative path for searching files
        :return: list of all Excel fils name (path) or list of 0 element if no files qualified
        '''

        pass

class AbstractObf(ABC):
    '''
    An abstract class designed for testing any obfuscation (fuzzy)
    This will be implemented by classes with concrete methods
    '''

    @staticmethod
    @abstractmethod
    def compare(string1: str, string2: str) -> int:
        '''
        compare two strings

        :param string1: string to check obfuscation
        :param string2: another string to check obfuscation
        :return: the number of edit distance as integer
        '''

        pass

    @staticmethod
    @abstractmethod
    def similarity(string1: str, string2: str, ratio: float) -> bool:
        '''
        call `compare` function to compare strings
        compare to ratio and decide whether this count as obfuscation (or not)

        **This function does NOT guarantee similarity based on real meaning of word,
        since this is not semantic segmentation**

        :param string1: string to check obfuscation
        :param string2: another string to check obfuscation
        :param ratio: the ratio for counting obfuscation
        :return: if > ratio return True, count as similar string
        '''


# Custom error class

class CosmicExcelCheckerException(Exception):
    '''
    Base Exception Class for the module
    '''

    pass

class IncorrectFileTypeException(CosmicExcelCheckerException):
    '''
    Raised when received incorrect file type. E.g: try to use load_excel to load files that are not excel.
    '''

    pass

class UnknownREQNumException(CosmicExcelCheckerException):
    '''
    Raised if a single result summary file does not have specific req number
    '''

    pass

class RepeatedREQNumException(CosmicExcelCheckerException):
    '''
    Raised if a single result summary file has repeated requirement number
    '''

    pass

class SheetNotFoundException(CosmicExcelCheckerException):
    '''
    Raised if a sheet is not found by given name
    '''

    pass


import pandas as pd
from openpyxl import load_workbook

import openpyxl
import xlrd

def check_for_charts_graphs_images(sheet_name):
  workbook = openpyxl.load_workbook('171testcosmic.xls')
  sheet = workbook[sheet_name]
  print(sheet.tables)
  print(list(sheet.values))
  for t in list(sheet.values):
      for v in t:
          if v is not None:
              print(v)
  for row in sheet.iter_rows():
      for cell in row:
          if cell.value is not None:
              print(cell)
              print(cell.value)
  return False

# print(check_for_charts_graphs_images(sheet_name='系数表'))

def test():
    df = pd.read_excel('171testcosmic.xls', sheet_name='结算评估确认表', engine='xlrd', skiprows=(0,))

    from tabulate import tabulate
    print(tabulate(df))

    print(df)


test()

def st():
    df = pd.read_excel('st.xlsx')

    print(df)
    # print(df.loc['A', 'E'])

st()

import re

pattern = re.compile("\/(?P<req>[0-9]{1,4})\/faqi\/[0-9A-Za-z,&@#$%.\[\]{};']+\.xlsx$")

if pattern.match("/454/faqi/hello.xlsx"):
    print(pattern.groups)
    print(pattern.groupindex)
    print(pattern.flags)

for match in re.finditer(pattern, '/123/faqi/asida.xlsx'):
    print(match)
    print(match['req'])
    print(type(match['req']))

df = pd.read_excel('171testcosmic.xls', sheet_name='系数表', engine='xlrd')
num_s : pd.Series = df['数值']
print("last", num_s.iloc[-1])
print("last df", df.iloc[-1, -1])

# pd.read_excel("st.xlsx", sheet_name="asjdoasjodajosdj")  # ValueError
df = pd.read_excel('st.xlsx')
print(df)
print('-' * 50)
df.columns = df.iloc[0]
print(df)
print('-' * 50)
df = df.iloc[1:]
print(df)
print('-' * 50)
df = df.reset_index(drop=True)
print(df)
print(df.iloc[1, 2])

import xlrd
book = xlrd.open_workbook('st.xls', formatting_info=True)
sheet = book.sheet_by_name('Sheet1')
print(sheet)

rows = sheet.nrows
cols = sheet.ncols

for i in range(rows):
    for j in range(cols):
        # print('val', sheet.cell_value(rowx=i, colx=j))
        cell = sheet.cell(rowx=i, colx=j)
        xfx = sheet.cell_xf_index(rowx=i, colx=j)  # xf index of given cell, index into xf_list
        xf = book.xf_list[xfx]  # a list of xf class instances, each corresponding to an XF record
        bgx = xf.background.pattern_colour_index
        # print(bgx)

        print(i, j, cell.value, bgx)

        # if bgx == 13:
        #     print(i, j, cell.value, 'yellow')

'''
    xlrd xf
    0x08 - 0x0a: 8 fixed colour xf index
    0x18, 0x40: SYS WIN text colour for border lines
    0x19, 0x41: bg 
    
    standard color: num (decimal)
    dark red: 60
    red: 10
    orange: 51
    yellow: 13
    light green: 50
    green: 17
    light blue: 40
    blue: 30
    dark blue: 56
    purple: 36
    white: 9
    no fill: 64
'''

book = load_workbook('st.xlsx')
sheet = book['Sheet1']
print(type(sheet.columns))
print(sheet.columns)
for row in sheet.iter_rows():
    print(row, type(row))
    for cell in row:
        print(cell)
        color_in_hex : str = cell.fill.start_color.index
        print(cell.value, color_in_hex, type(color_in_hex))
        print(cell.row, cell.column)  # get index of the cell, 1-based

for i in range(len(list(sheet.rows))):
    row = list(sheet.rows)[i]
    print(row)

    for j in range(len(row)):
        cell = row[j]
        print(i, j, cell)

from deprecated import deprecated

@deprecated(version='0.1.0', reason='deprecated in 0.2.0')
def hello():
    print("hello")

hello()

a = float('0.33')
b = 1/3
if abs(a - b) < 0.01:
    print('e')
else:
    print('ne')








